##############################
#Multilevel workshop II Examples
#June 27/28, 2013
#StatLab@UVa Library
#Michele Claibourn
##############################

#Load packages
#install.packages("locfit")
library(lme4)
library(foreign)
library(lattice)
library(locfit)

#Load data
schools <- read.dta('http://www.people.virginia.edu/~mpc8t/Multilevel2Workshop/schools.dta')

##The Linear Mixed-Effects Model
school2.lme <- lmer(mathach ~ catholic*cses + meanses*cses + (cses | school), data=schools)
summary(school2.lme)


##Exploratory Data Analysis and Model Building
#Estimate separate regressions for each group
school.lis <- lmList(mathach ~ cses + female + minority | school, data=schools)
summary(school.lis)

#Caterpillar plot
plot(confint(school.lis))

#Scatterplots
#By groups, using lattice plots
#First create a subset of data with only 25 schools; the graph is too big/slow with all 160 schools
schools.25 <- subset(schools, school<2500)
xyplot(mathach ~ cses | school, data=schools.25, main="Math Achievement by SES",
       panel=function(x, y){
         panel.xyplot(x,y)
         panel.loess(x,y,span=1)
         panel.lmline(x,y,lty=2)}
)

xyplot(mathach ~ minority | school, data=schools.25, main="Math Achievement by Minority",
       panel=function(x, y){
         panel.xyplot(x,y)
         panel.loess(x,y,span=1)
         panel.lmline(x,y,lty=2)}
)

#By categorical groupl-level covariate
xyplot(mathach ~ cses | catholic, data=schools, 
       panel=function(x,y){
         panel.xyplot(x,y)
         panel.loess(x,y,span=1)
         panel.lmline(x,y,lty=2)}
)

xyplot(mathach ~ female | catholic, data=schools, 
       panel=function(x,y){
         panel.xyplot(x,y, jitter.x=TRUE)
         panel.loess(x,y,span=1)
         panel.lmline(x,y,lty=2)}
)

#By continuous group-level covariate
# Conditioning plots (coplots) create shingles for a graph. 
# John Fox's supplement, panel=panel.car, automatically adds a regression line and lowess line.
#Load the car library
library(car)

#Recreate the graph of math achievement by ses conditioning on catholic
coplot(mathach ~ cses | catholic, panel=panel.car, data=schools, rows=1)

#The coplot function will automatically categorize a continuous variable.
#The number option determines how many categories are created.
#The overlap option determines the proportion of cases that overlap shingles.
coplot(mathach ~ cses | meanses, panel=panel.car, data=schools, number=5, overlap=.1, rows=1)


##Distributions
#We could run the regressions and collect the regression objects into a dataframe for graphing
schoolregs <- lapply(unique(schools$school), function(x) {
  lm(mathach ~ cses , data = schools[schools$school == x, ])
})
names(schoolregs) <- unique(schools$school)
coefmatrix <- matrix(unlist(lapply(schoolregs, coef)), ncol = 2, byrow = TRUE, dimnames = list(names(schoolregs), c("Intercept", "cses")))
sematrix <- matrix(unlist(lapply(schoolregs, function(x) summary(x)$coef[, "Std. Error"])), ncol = 2, byrow = TRUE, dimnames = list(names(schoolregs), c("SEIntercept", "SEcses")))
coefse <- data.frame(coefmatrix, sematrix)
coefse$school <- factor(row.names(coefse))

#Level-1 distribution (y, epsilon)
hist(schools$mathach, main = "Distribution of Y")

#Random coefficients (beta)
plot(density(coefse$Intercept), main = "Density of Intercepts")
rug(coefse$Intercept)

qqnorm(coefse$cses, main = "Are Slopes Approximately Normal?")
qqline(coefse$cses)

#Bivariate density/contour plot
plot(coefse$Intercept, coefse$cses, pch = 19, col = "black", xlab = "Intercepts", ylab = "Slopes", cex.lab = 1)
plot(locfit(~Intercept * cses, data = coefse, scale = T, kern = "rect", deg = 2, family = "dens"), 
     add = TRUE, col = gray(0.5), drawlabels = FALSE)


##Estimation
#Full ML and REML
school2.fml <- lmer(mathach ~ catholic*cses + meanses*cses + (cses | school), REML = FALSE, data=schools)
summary(school2.fml)

school2.reml <- lmer(mathach ~ catholic*cses + meanses*cses + (cses | school), REML = TRUE, data=schools)
summary(school2.reml)

#The BLUPs
dotplot(ranef(school2.lme, postVar = TRUE))


##Inference
#Fixed effecs
#Using linearHypothesis command in car package
library(car)
#Are a set of level-2 predictors for a particular level-1 parameter jointly zero
linearHypothesis(school2.lme, c("catholic:cses = 0", "cses:meanses = 0"))

#Is cses significant within Catholic schools?
linearHypothesis(school2.lme, c("cses + catholic:cses = 0"))

#Doug Bates, author of lme4, cautions against the use of p-values, but suggests LR tests for fixed effects.
#See: https://stat.ethz.ch/pipermail/r-help/2006-May/094765.html

#Variance components
#Is Random intercept significant?
#Bates is even less enthusiastic about relying on p-values for random effects stating:
# "the theory beyind using a chi^2 distribution with 1 degree of freedom as a reference distribution for the test statistic does not apply in this case. 
# "The null hypothesis is on the boundary of the parameter space...Even at the best of times, the p-values for such tests are only approximate."
school2.rim <- lmer(mathach ~ catholic*cses + meanses*cses + (1 | school), data=schools)
school2.lm <- lm(mathach ~ catholic*cses + meanses*cses, data=schools)

dev <- as.numeric(2*(logLik(school2.rim)-logLik(school2.lm)))
lrt.rim <- 0.5*(1-pchisq(dev, 1))
lrt.rim

#Is covariance between random intercept and slope significant?
school2.rcoef <- lmer(mathach ~ catholic*cses + meanses*cses + (cses | school), data=schools)
school2.rcoefnocovar <- lmer(mathach ~ catholic*cses + meanses*cses + (1 | school) + (0 + cses | school), data=schools)
anova(school2.rcoef, school2.rcoefnocovar)

#Is random slope significant
anova(school2.rcoefnocovar, school2.rim)
anova(school2.rcoef, school2.rim)

#Variance parameters near zero (s.e. calculations don't fail for lme4 as they do in Stata)
school3.lme1 <- lmer(mathach ~ catholic*cses + meanses*cses + female + minority + (cses + female + minority | school), data=schools)
summary(school3.lme1)
school3.lme2 <- lmer(mathach ~ catholic*cses + meanses*cses + female + minority + (female + minority | school), data=schools)
summary(school3.lme2)


#Model Comparison, Assessment
#Error assumptions, level-1
qqPlot(residuals(school2.lme, type='pearson'))

#Random effects, level-2
re <- as.data.frame(ranef(school2.lme)[1])
randints <- re[,1]
randslopes <- re[,2]

qqPlot(randints)
qqnorm(randslopes)
qqline(randslopes)

#Bivariate density/contour plot
plot(randints, randslopes, pch = 19, col = "black", xlab = "Intercepts", ylab = "Slopes", cex.lab = 1)
plot(locfit(~randints * randslopes, scale = T, kern = "rect", deg = 2, family = "dens"), 
     add = TRUE, col = gray(0.5), drawlabels = FALSE)


##Generalizations
#Load Hospital-Doctor-Patient simulated data
hdp <- read.dta('http://www.people.virginia.edu/~mpc8t/Multilevel2Workshop/hdp.dta')
dim(hdp)
head(hdp)
summary(hdp)

#Three-level model
mobility1 <- lmer(mobility ~ age + sex + bmi + (1|did) + (1|hid), data=hdp)
summary(mobility1)

mobility2 <- lmer(mobility ~ age + sex + bmi + experience + school+ medicaid + (1|did) + (1|hid), data=hdp)
summary(mobility2)

#Mixed-effects logit
remission <- glmer(remission ~ age + lengthofstay + il6 + familyhx + cancerstage + experience + 
                     (1 | did), data = hdp, family = binomial)
summary(remission)
